(* $Id: indexed.mli 1 2005-07-15 19:39:24Z lemaire $ *)
(** 
 The class for indexed objects (observations, patterns, ...)
  
 Indexing an object is intended to be able to recover an original
 ordering after several mixes or sorting may have happen. It is also
 a manner to give a 'name' to object to identify them (useful when
 subsets are considered).

 @version November 08, 2004
 @author Pierre Lemaire
*)

(* ============================================================================
author: Pierre Lemaire
2004-11-01             : creation
2004-11-08             : Indexed.compare
============================================================================ *)

(* ======================================================================= *)
(** {6 the class [indexed]}*)

(** The class [indexed] permits every class that extends it to have an index.

    Typically, it is the index of an observation in an instance, of a pattern
    in a model, etc.
  
  *)
class indexed:
  int -> (* the initial index *)
object
  (* ===================================================================== *)
  (** {6 fields} *)
    
  (** an index ([-1] if not specified). *)
  val mutable index: int

  (* ===================================================================== *)
  (** {6 properties} *)

  (** returns the index of this object. *)
  method index: int
  
  (** sets the index of this object. *)
  method set_index: int -> unit

end

(* ======================================================================= *)
(** {6 comparisons}*)

(** compares 2 [indexed] according to their index. *)
val compare: indexed -> indexed -> int

