(* $Id: instance.ml 11 2007-02-28 09:38:48Z pierre $ *)
(* ============================================================================
author: Pierre Lemaire
2004-10-29             : creation
2004-10-29..2004-11-08 : basics
============================================================================ *)

open Printf;;
open Messages;;
open Observation;;

(* =========================================================================
   class instance
   ========================================================================= *)
class instance _kmax _obs_init = 
object
  (* =======================================================================
     fields
     ======================================================================= *)
  val mutable obs = _obs_init
  val kind = Array.make (_kmax+1) 0;  
  val mutable undef = 0;
  val mutable n = (-1);
  
  initializer
    for i=0 to Array.length _obs_init - 1 do
      let knew, nnew = _obs_init.(i)#kind, _obs_init.(i)#n
      in if (n < nnew) then n <- nnew;
         if (knew < 0) then undef <- undef + 1
         else kind.(knew) <- kind.(knew) + 1;
    done

  (* =======================================================================
     counters
     ======================================================================= *)
  
   method k = _kmax + 1
   method count_kinds = _kmax + 1
     
  (* ===================================================================== *)
   method m = Array.length obs
   method count_observations = Array.length obs

  (* ===================================================================== *)
   method n = n
   method count_variables = n
   
  (* ===================================================================== *)
   method count_of_kind i =
     try kind.(i)
     with
     | Invalid_argument "index out of bounds"
     -> invalid_arg "Instance.count_of_kind: no such kind"

  (* ===================================================================== *)
   method count_undefined = undef
   
  (* =======================================================================
     properties
     ======================================================================= *)

   method observation i =
     try obs.(i)
     with
     | Invalid_argument "index out of bounds"
     -> invalid_arg "Instance.observation: no such observation"
   
   method set_observation i o =
     try
       let (kold, knew, nnew) = obs.(i)#kind, o#kind, o#n
       in if (n < nnew) then n <- nnew;
          if (kold <> knew) then begin
            try 
              if (kold < 0) then undef <- undef - 1
              else kind.(kold) <- kind.(kold) - 1;
              if (knew < 0) then undef <- undef + 1
              else kind.(knew) <- kind.(knew) + 1;
            with 
            | Invalid_argument "index out of bounds"
            -> invalid_arg "Instance.set_observation: no such kind"
          end;
          obs.(i) <- o
     with
     | Invalid_argument "index out of bounds"
     -> invalid_arg "Instance.set_observation: no such observation"

  (* =======================================================================
     methods
     ======================================================================= *)
   method normalize () =
     for v = 0 to n-1 do
       let sum = ref 0.0
       and sumsq = ref 0.0
       in
         for o=0 to Array.length obs -1 do
           sum := !sum +. obs.(o)#value v;
           sumsq := !sumsq +. (obs.(o)#value v)*.(obs.(o)#value v);
         done;
         let m = float_of_int (Array.length obs)
         in let ave   = !sum /. m;
            and stdev = sqrt(((!sumsq)-.(!sum *. !sum)/.m)/.(m-.1.0))
         in for o=0 to Array.length obs -1 do
           let x = obs.(o)#value v
           in obs.(o)#set_value v ((x-.ave)/.stdev)
         done;
     done;
        
  (* =======================================================================
     sorting
     ======================================================================= *)

   method sort () = Array.stable_sort (Observation.compare) obs 
   
  (* ===================================================================== *)
   method mixup () =
     let m = (Array.length obs)
     in let tmp = Array.make m (empty_observation)
     and ok  = Array.make m false
     in for i=m downto 1 do
       let k = Random.int i
       and j = ref 0
       and c = ref 0
       in while (!c < k) || (ok.(!j)) do
         if not ok.(!j) then c := !c + 1;
         j := !j + 1;
         done;
         tmp.(i-1) <- obs.(!j); ok.(!j) <- true;
       done;
       obs <- tmp;

end;;
