(* $Id: io.mli 1 2005-07-15 19:39:24Z lemaire $ *)
(** 
 Functions for i/o of data structures.

 @version November 08, 2004
 @author Pierre Lemaire
*)

(* ============================================================================
author: Pierre Lemaire
2004-11-05             : creation
2004-11-05..2004-11-08 : observation, instance
============================================================================ *)

open Misc;;
open Observation;;
open Instance;;
open Monome;;
open Pattern;;
open Pandect;;
open Model;;
open Cutpointset;;


(* ======================================================================= *)
(** {6 i/o for [observations]} *)

(** reads a fresh [observation] from the given buffer. A [IO_error] is raised
    if a problem occurs. *)
val read_observation: Lexing.lexbuf -> observation

(** reads a fresh [observation] from the given string. A [IO_error] is raised
    if a problem occurs. *)
val observation_of_string: string -> observation

(** returns a [string] representation of the [observation]. *)
val string_of_observation: observation -> string

(** prints a representation of the [observation] on the given channel. *)
val print_observation: out_channel -> observation -> unit

(* ======================================================================= *)
(** {6 i/o for [instances]} *)

(** reads a fresh [instance] from the given buffer. A [IO_error] is raised
    if a problem occurs. *)
val read_instance: Lexing.lexbuf -> instance

(** reads a fresh [instance] from the given string. A [IO_error] is raised
    if a problem occurs. *)
val instance_of_string: string -> instance

(** returns a [string] representation of the [instance]. *)
val string_of_instance: instance -> string

(** prints a representation of the [instance] on the given channel. *)
val print_instance: out_channel -> instance -> unit

(* ======================================================================= *)
(** {6 i/o for [...]} *)

val string_of_monome: monome -> string
val print_monome: out_channel -> monome -> unit
val print_pattern: out_channel -> pattern -> unit
val read_pandect: ?noer:bool -> Lexing.lexbuf -> pandect
val read_model: ?noer:bool -> Lexing.lexbuf -> model
val print_pandect: out_channel -> pandect -> unit

val read_cuts: Lexing.lexbuf -> cutpointset list
val print_cutpointset: out_channel -> cutpointset -> unit
val print_cutpointset_list: out_channel -> cutpointset list -> unit

