(* $Id: misc.ml 1 2005-07-15 19:39:24Z lemaire $ *)
(* ============================================================================
author: Pierre Lemaire
2004-10-26             : creation
============================================================================ *)

open Messages;;

open Printf;;
open Lexing;;

(* ========================================================================= *)
let id x = x;;

let set_ref r v = r := v;;

(* ========================================================================= *)
let eol = ref "\n";;

let eol_options = 
  [("-noeol",
      Arg.Unit (fun () -> eol := ""),
      " no EOL.");
   ("-unixeol",
      Arg.Unit (fun () -> eol := "\n"),
      " unix-like EOL.");
   ("-doseol",
      Arg.Unit (fun () -> eol := "\r\n"),
      " dos-like EOL.");
   ("-maceol",
      Arg.Unit (fun () -> eol := "\r"),
      " mac-like EOL.");
  ]
;;
  
(* ========================================================================= *)
let common_options x = let (name,version,description,author,copyright) = x in
  [("-version",
      Arg.Unit  (fun () -> fprintf stdout "%s\n" (name^", "^version)),
      " prints the version.");
   ("-about",
      Arg.Unit (fun () -> fprintf stdout "%s\n"
       (name^": "^description^"\n"
	^"Copyright (C) "^author^", "^copyright^".\n\n"
	^"Please, report remarks or bugs at lemaire@rutcor.rutgers.edu\n"
	^"This program is free software; you can redistribute it and/or\n"
	^"modify it under the terms of the GNU General Public License\n"
	^"as published by the Free Software Foundation; either version 2\n"
	^"of the License, or (at your option) any later version.\n"
	^"This program is distributed in the hope that it will be useful,\n"
	^"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	^"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the\n"
	^"GNU General Public License for more details.\nThe "
	^"GNU GPL is available at http://www.gnu.org/copyleft/gpl.html\n")),
      " prints a copyright notice.");
   ("-nan",
      Arg.Unit (fun () -> fprintf stdout "nan=%f\n" nan),
      " prints the value for [nan]");
   ("-o",
      Arg.String (set_output_file),
      ("(file) sets the output file (default: stdout)"));
   ("-a",
      Arg.String (set_output_file ~append:true),
      ("(file) same as -o for append mode"));
   ("-trace",
      Arg.String (set_trace_level_from_string),
      ("(val) sets the verbosity [none|some|most|all] of the trace "
       ^"(default: none)"));
   ("-ftrace",
      Arg.String (set_trace_file),
      "(file) sets where the trace is printed (default: stderr)");
   ("-aftrace",
      Arg.String (set_trace_file ~append:true),
      "(file) same as -ftrace for append mode");
   ("-fwarning",
      Arg.String (set_warning_file),
      "(file) sets where the warnings are printed (default: stderr)");
   ("-afwarning",
      Arg.String (set_warning_file ~append:true),
      "(file) same as -fwarning for append mode");
   ("-ferror",
      Arg.String (set_error_file),
      "(file) sets where the errors are printed (default: stderr)");
   ("-aferror",
      Arg.String (set_error_file ~append:true),
      "(file) same as -ferror for append mode");
   ("-debug",
      Arg.Unit (fun () -> set_trace_level TRACE_DEBUG),
      " debug mode (a lot of trace)");
   ("-names",
    Arg.Unit (fun () -> set_names true),
    " prints names (row, columns, observations...).");
   ("-nonames",
    Arg.Unit (fun () -> set_names false),
    " does not print names.");
  ]
;;

