(* $Id: monome.ml 11 2007-02-28 09:38:48Z pierre $ *)
(* ============================================================================
author: Pierre Lemaire
2004-11-01             : creation
2004-11-01..2004-11-03 : basics
============================================================================ *)

open Printf;;
open Observation;;

type inequality_type = LT of float | GT of float | BOX of float*float;;

(* =======================================================================
   the class monome
   ======================================================================= *)

class monome _variable _inequality =
object(self)

  (* =======================================================================
     properties
     ======================================================================= *)

  (* ===================================================================== *)
  method variable = _variable

  (* ===================================================================== *)
  method inequality = _inequality
  
  (* ===================================================================== *)
  method ub = match _inequality with
    | LT x    -> x
    | GT _    -> infinity
    | BOX (_,x) -> x
  
  (* ===================================================================== *)
  method lb = match _inequality with
    | LT _    -> neg_infinity
    | GT x    -> x
    | BOX (x,_) -> x
  
  (* ===================================================================== *)
  method covers (o: observation) =
    let v = o#value _variable
    in if (Pervasives.compare v nan = 0) then failwith "undefined"
    else match _inequality with
      | LT x      -> Pervasives.compare (v+.epsilon_float) x <= 0
      | GT x      -> Pervasives.compare (v-.epsilon_float) x >= 0
      | BOX (x,y) ->    Pervasives.compare (v-.epsilon_float) x >= 0
                     && Pervasives.compare (v+.epsilon_float) y <= 0

  (* ===================================================================== *)
  method covers_with_epsilon e (o: observation) =
    let e = if self#covers o then e else -. e
    and v = o#value _variable
    in if (Pervasives.compare v nan = 0) then false
    else match _inequality with
      | LT x      -> Pervasives.compare (v+.epsilon_float) (x-.e) <= 0
      | GT x      -> Pervasives.compare (v-.epsilon_float) (x+.e) >= 0
      | BOX (x,y) ->    Pervasives.compare (v-.epsilon_float) (x+.e) >= 0
                     && Pervasives.compare (v+.epsilon_float) (y-.e) <= 0
end;;


(* adds a monome x to the list of monomes *)
(* every monome is assumed to be of the form GT _ *)
let rec add_gt_monome ms x = 
  match ms with
    | [] -> [x]
    | m::ms ->
        if x#variable < m#variable then x::m::ms
        else if x#variable > m#variable then m::(add_gt_monome ms x)
        else if x#lb < m#lb then x::m::ms
        else if x#lb > m#lb then m::(add_gt_monome ms x)
        else m::ms
;;
