(* $Id: monome.mli 1 2005-07-15 19:39:24Z lemaire $ *)
(** 
 The class for a monome.

 @version November 8, 2004
 @author Pierre Lemaire
*)

(* ============================================================================
author: Pierre Lemaire
2004-11-01             : creation
2004-11-01..2004-11-08 : basics
============================================================================ *)

open Observation;;

(** Type for the different inequalities for a [monome].
    The inequalities are [LX x] (less than x), [GT x] (greater than x)
    and [BOX x y] (in \[x..y\]). *)
type inequality_type = LT of float | GT of float | BOX of float*float;;

(* ========================================================================= *)
(** {6 the class [monome]}  *)

(** The class [monome] is the basic component of a  [pattern].

    [monome i ineq] creates a new monome for variable [i] and inequality [ineq].
*)
class monome:
  int ->
  inequality_type ->
object

  (* ===================================================================== *)
  (** {6 properties} *)
  
  (** returns the variable index this [monome] is defined on. *)
  method variable: int

  (** returns the inequality type this [monome] is defined with. *)
  method inequality: inequality_type
  
  (** returns the upper-bound this [monome] uses ([infinity] if none). *)
  method ub: float

  (** returns the lower-bound this [monome] uses ([neg_infinity] if none). *)
  method lb: float

  (* ===================================================================== *)
  (** {6 methods} *)
      
  (** returns whether this [monome] covers the given [observation] or not.
    raise [Failure "undefined"] if the value is not defined.*)
  method covers: observation -> bool
  
  (** returns whether this [monome] covers the given [observation] or not, with
      a 'confidence' of at least [e]. *)
  method covers_with_epsilon: float -> observation -> bool

end

(** adds a [monome] to the given list, without duplicate. 
    note: every monome is assumed to be of the form GT _ *)
val add_gt_monome: monome list -> monome -> monome list
