(* $Id: observation.ml 11 2007-02-28 09:38:48Z pierre $ *)
(* ============================================================================
author: Pierre Lemaire
2004-10-26             : creation
2004-10-26..2004-11-08 : basics
============================================================================ *)

open Printf;;
open Messages;;
open Indexed;;

(* =========================================================================
   class observation
   ========================================================================= *)
class observation
  ?(_name = "_")            (* name [should be non-empty] *)
  (_index: int)             (* index *)
  (_kind: int)              (* kind/class *)
  (_value: float array) =   (* values of the variables *)
object
  inherit indexed _index
  
  val mutable name = _name;
    
  (* =======================================================================
     properties
     ======================================================================= *)

  (* ===================================================================== *)
  method name = name;
  
  (* ===================================================================== *)
  method set_name s = name <- s;

  (* ===================================================================== *)
  method kind = _kind;
  
  (* ===================================================================== *)
  method n = Array.length _value;
       
  (* ===================================================================== *)
  method value i =
    try _value.(i);
    with Invalid_argument "index out of bounds"
    -> (*invalid_arg "Observation.value: no such variable"*) nan
    
  (* ===================================================================== *)
  method set_value i x = _value.(i) <- x
  
  (* ===================================================================== *)
  method clone () = 
    let tmp = Array.copy _value in new observation _index _kind tmp
  
  (* ===================================================================== *)
  method distance (o: observation) =
    let d = ref 0.0
    in for i = 0 to Array.length _value - 1 do
       let x = _value.(i) -. (o#value i) in d := !d +. (x *. x);
      done;
      sqrt !d

end;;

(* ========================================================================= *)
let empty_observation = new observation (-1) (-1) [||];;

(* =========================================================================
   comparison
   ========================================================================= *)

(* ========================================================================= *)
let compare_index o1 o2 = Indexed.compare (o1:>indexed) (o2:>indexed);;

(* ========================================================================= *)
let compare_name o1 o2 = String.compare (o1#name) (o2#name);;
   
(* ========================================================================= *)
let compare_all o1 o2 =
  let c = Pervasives.compare o1#kind o2#kind
  in
    if c != 0 then c
    else try
      for i=0 to o1#n-1 do
        let c = Pervasives.compare (o1#value i) (o2#value i)
        in if (c<0) then failwith "1" else if (c>0) then failwith "2";
      done;
    0;
    with
    | Failure "1" -> -1
    | Failure "2" ->  1
;;
   
(* ========================================================================= *)
let compare_kind o1 o2 = Pervasives.compare o1#kind o2#kind;;

(* ========================================================================= *)
let compare_variable i o1 o2 = Pervasives.compare (o1#value i) (o2#value i);;
   
(* ========================================================================= *)
let compare_function = ref compare_kind;; (* private and hidden *)
let compare o1 o2 = !compare_function o1 o2;;

(* ========================================================================= *)
let set_compare_function f = compare_function := f;;
